import { Component, OnInit, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { CustomInputComponent } from './../../_abstract/custom-input.component';
import { ValueTransformer }	from './../../../interfaces';

@Component({
	selector: 'app-dropdown-modified-input',
	templateUrl: 'dropdown-modified-input.component.html',
	styles: [],
	providers: [
		{
			provide: NG_VALUE_ACCESSOR,
			useExisting: forwardRef(() => DropdownModifiedInputComponent),
			multi: true
		}
	]
})
export class DropdownModifiedInputComponent extends CustomInputComponent implements OnInit {

	exposeMetaInTemplate: string[] = ['modifiers', 'transform', 'placeholder'];

	modifiers: string[];
	transform: ValueTransformer = {
		inputFn: value => ({ modifier: '', value }),
		outputFn: (modifier, value) => value
	};
	private selectedModifier: string;
	private displayedValue: string;
	private _controlValue: string;

	ngOnInit() {
		super.ngOnInit();
		this.controlValue = this.control.value;
	}

	get controlValue(): string {
		return this._controlValue;
	}

	set controlValue(inputValue) {
		const { modifier, value } = this.transform.inputFn(inputValue);
		this.selectedModifier = modifier;
		this.displayedValue = value;
		this._controlValue = inputValue;
	}

	writeValue(value: string): void {
		this.controlValue = value;
	}

	modifierChange(modifier) {
		this.selectedModifier = modifier;
		this._controlValue = this.transform.outputFn(this.selectedModifier, this.displayedValue);
		this.propagateChange(this.controlValue);
	}

	inputChange() {
		this._controlValue = this.transform.outputFn(this.selectedModifier, this.displayedValue);
		this.propagateChange(this.controlValue);
	}

}
